@@ -142,7 +142,10 @@ namespace Mono.WebServer.FastCgi
 
 		public override string GetPathInfo ()
 		{
-			return responder.GetParameter ("PATH_INFO");
+			string ret = responder.GetParameter ("PATH_INFO");
+			if (ret == null)
+				return String.Empty;
+			return ret;
 		}
 		
 		public override string GetRawUrl ()
