@@ -185,6 +185,9 @@ namespace Mono.WebServer
 					b = inputBuffer [inputLength - 1];
 					if (b != '\r' && b != '\n')
 						continue;
+					FillBuffer();
+					if (b == '\r' && inputLength > 0 && inputBuffer[0] == '\n')
+						position++;
 				}
 				break;
 			} while (true);
@@ -239,7 +242,6 @@ namespace Mono.WebServer
 				path = path.Substring (0, qmark);
 			}
 
-			path = HttpUtility.UrlDecode (path);
 			path = GetSafePath (path);
 			if (path.StartsWith ("/~/")) {
 				// Not sure about this. It makes request such us /~/dir/file work
