@@ -280,6 +280,9 @@ namespace Mono.WebServer
 					b = inputBuffer [inputLength - 1];
 					if (b != '\r' && b != '\n')
 						continue;
+					FillBuffer();
+					if (b == '\r' && inputLength > 0 && inputBuffer[0] == '\n')
+						position++;
 				}
 				break;
 			} while (true);
@@ -730,6 +733,23 @@ namespace Mono.WebServer
  			return secure;
  		}
 
+		public override void SendResponseFromFile (string filename, long offset, long length)
+		{
+			using (FileStream fs = File.OpenRead (filename)) {
+					if (secure || no_libc || (tried_sendfile && !use_sendfile)) {
+					// We must not call the SendResponseFromFile overload which
+					// takes  IntPtr in this case since it will call the base
+					// implementation of that overload which, in turn, will
+					// close the handle (as it uses FileStream to wrap the
+					// handle we pass). This will cause the handle to be closed
+					// twice (FileStream owns the handle). So we just take a
+					// shortcut to what the base overload does here.
+					SendFromStream (fs, offset, length);
+				} else
+					SendResponseFromFile (fs.Handle, offset, length);
+			}
+		}
+
 		public override void SendResponseFromFile (IntPtr handle, long offset, long length)
 		{
 			if (secure || no_libc || (tried_sendfile && !use_sendfile)) {
@@ -743,7 +763,7 @@ namespace Mono.WebServer
 				Cork (true);
 				SendHeaders ();
 				while (length > 0) {
-					result = sendfile (socket, handle, ref offset, (IntPtr) length);
+					result = sendfile ((int) socket, (int) handle, ref offset, (IntPtr) length);
 					if (result == -1)
 						throw new System.ComponentModel.Win32Exception ();
 
@@ -773,7 +793,7 @@ namespace Mono.WebServer
 				return 0;
 			// 6 -> SOL_TCP, 3 -> TCP_CORK
 			bool t = val;
-			return setsockopt (socket, (IntPtr) 6, (IntPtr) 3, ref t, (IntPtr) IntPtr.Size);
+			return setsockopt ((int) socket, 6, 3, ref t, (IntPtr) IntPtr.Size);
 		}
 
 		unsafe int Send (byte [] buffer, int offset, int len)
@@ -787,7 +807,7 @@ namespace Mono.WebServer
 			while (total < len) {
 				fixed (byte *ptr = buffer) {
 					// 0x4000 no sigpipe
-					int n = send (socket, ptr + total, (IntPtr) (len - total), (IntPtr) 0x4000);
+					int n = send ((int) socket, ptr + total, (IntPtr) (len - total), (int) 0x4000);
 					if (n >= 0) {
 						total += n;
 					} else if (Marshal.GetLastWin32Error () != 4 /* EINTR */) {
@@ -803,13 +823,13 @@ namespace Mono.WebServer
 		static bool use_sendfile;
 
 		[DllImport ("libc", SetLastError=true)]
-		extern static int setsockopt (IntPtr handle, IntPtr level, IntPtr opt, ref bool val, IntPtr len);
+		extern static int setsockopt (int handle, int level, int opt, ref bool val, IntPtr len);
 
 		[DllImport ("libc", SetLastError=true)]
-		extern static int sendfile (IntPtr out_fd, IntPtr in_fd, ref long offset, IntPtr count);
+		extern static int sendfile (int out_fd, int in_fd, ref long offset, IntPtr count);
 
 		[DllImport ("libc", SetLastError=true, EntryPoint="send")]
-		unsafe extern static int send (IntPtr s, byte *buffer, IntPtr len, IntPtr flags);
+		unsafe extern static int send (int s, byte *buffer, IntPtr len, int flags);
 	}
 }
 
